#!/usr/sbin/perl

########################################################################
# grouper.perl
# create group tags based on regexp input
#  groups are only based on values of a single tag
#  can have more than one set of arguments
#   (see linegrouper.perl for grouping based on multiple tags)
#
# input: NDP format (lines of name/value pairs on stdin)
#         "tag=value" "name={~|\!}regexp" sets of arguments on command line
#	(note that you need "=\!" instead of just "=!" to get it by shell..)
# output: NDP format, adding "tag=value" pair for lines where 
#          "name" matches the regular expression if pair was "name=~regexp"
#	     or name doesn't match if pair was "name=!regexp".
#  
# usage: can colorize based on tag values, can declare groups which
#        geometry converters can use to create separate objects, etc.
#
# tamara munzner
# 6/96
########################################################################

$i = 0;
while (@ARGV) {
    undef($tag, $value, $name, $regexp);
    ($tag,$value) = split('=', shift(@ARGV));
    ($name,$regexp) = split('=', shift(@ARGV));
    $comps[$i] = substr($regexp,0,1);
    $regexp = substr($regexp,1);
    $namedef{$name} = 1;
    $names[$i] = $name;
    $regexps[$i] = $regexp;
    $tags[$i] = $tag;
    $values[$i] = $value;
    $i++;
}


while (<STDIN>) {
    # deal with a series of items that look like this:
    # ( "name=val", "name2=val2", ...)
    next if $_ eq "\n";
    chop;
    # note that can't do a split(',',$_)!
    # that would split placenames...
    # treat the line as an array, it's ( a, b, c, d)
    @line = eval($_);
    print "(";
    foreach $item (@line) {
	undef($lhs, $rhs);
	print " \"$item\",";
	($lhs,$rhs) = split('=', $item);
	if (defined($namedef{$lhs})) {
	    foreach $j (0..$i-1) {
		if ($names[$j] eq $lhs) {
		    $tmprhs = $rhs;
		    if ($comps[$j] eq "~") {
			print " \"$tags[$j]=$values[$j]\", " 
			    if ($tmprhs =~ /$regexps[$j]/);
		    } elsif ($comps[$j] eq "!") {
			print " \"$tags[$j]=$values[$j]\", " 
			    if ($tmprhs !~ /$regexps[$j]/);
		    }
		}
	    }
	}
    }
    print ")\n";
}
	
