#!/usr/local/bin/perl

########################################################################
# hostresolve.perl:
#  input: lines of name/value pairs
#  output: same lines, with possible additional name/value pairs 
#   {src|dst}{lat|lon|place}:
#   latitudes, longitudes, and placenames if
#   we find the values in our hostname->placename->latlong databases
# tamara munzner
# 4/96
########################################################################

$netinfodir = "/u/munzner/ndp/netinfo";

open(JUNK, ">>junk.res");

&snarffiles();

while (<STDIN>) {
    # deal with a series of items that look like this:
    # ( "name=val", "name2=val2", ...)
    undef($srcip, $dstip, $srchost, $dsthost);
    undef(%items, @line);
    chop;
    @line = split(',');
    foreach $item (@line) {
	undef($lhs, $rhs);
	$item =~ /\(?\s*\"([^=]*)=([^\"]*)\"/;
	$lhs = $1; $rhs = $2;
	if (defined($lhs) && defined($rhs)) {
	    eval "\$$lhs = \"$rhs\";";
	}
	$items{$lhs} = $rhs;
    }
    # we may overwrite certain fields. 
    # since we're sorting the keys, we know that *{lat|lon|place}
    #  will arrive after *host, so our method is OK.
    undef($srclat, $srclon, $srcplace);
    undef($dstlat, $dstlon, $dstplace);
    undef(@overwrite);
    print "( ";
    foreach $lhs (sort keys %items) {
	$rhs = $items{$lhs};
	if ($lhs eq "srchost") {
	    print " \"srchost=$rhs\",";
	    ($srclat, $srclon, $srcplace) = &resolve($rhs);
	    if ($srcplace != -1) {
		printf " \"srclat=%4.3f\", \"srclon=%4.3f\", \"srcplace=%s\", ", 
		$srclat, $srclon, $srcplace;
		push(@overwrite,"(srclat, srclon, srcplace)");
	    }
	} elsif ($lhs eq "dsthost") {
	    print " \"dsthost=$rhs\",";
	    ($dstlat, $dstlon, $dstplace) = &resolve($rhs);
	    if ($dstplace != -1) {
		printf " \"dstlat=%4.3f\", \"dstlon=%4.3f\", \"dstplace=%s\", ",
		$dstlat, $dstlon, $dstplace;
		push(@overwrite,"(dstlat, dstlon, dstplace)");
	    }
	} elsif (!(scalar(grep(/$lhs/,@overwrite)))) {
	    print " \"$lhs=$rhs\",";
	}
    }
    print ") \n";
}


sub snarffiles {
    local($place, $lat, $long, $name, $host);

    open(P2L, "<$netinfodir/place2latlong");
    while (<P2L>) {    
	s/\s*#.*$//;
	next if /^\s*$/;
	($place, $lat, $long) = /^(.*)\s+([-0-9.]+)\s+([-0-9.]+)\s*$/o;
	$place =~ s/\s+$//o;
	$p2l{"\L$place\E"} = join(";",$lat,$long);
    }
    close(P2L);

    open(H2P, "<$netinfodir/name2place");
    while (<H2P>) {
	s/#.*$//;
	next if /^\s*$/;
	($name, $place) = split(/\s+/,$_,2);
	chop($place);
	$place =~ s/\s+$//o;
	if ($p2l{"\L$place\E"}) {
		$hostdata{"\L$name\E"} = join(';',$p2l{"\L$place\E"},$place);
	} else {
		print JUNK "nolatlongfor $place\n";
	}
    }
    close(H2P);

    open(LATLONGS, ">latlongs");
    foreach $host (sort keys(%hostdata)) {
	printf LATLONGS "%s\t %f\t %f\t %s\n", $host, split(';',$hostdata{$host});
    }
    close(LATLONGS);
}

sub longestmatch {
    local($hostname) = @_;
    undef(local($info, $i, @host, $index));
    @host = split('\.', "\L$hostname\E");
    while ($i <= $#host) {
	$index = join('.',@host[$i++..$#host]);
	$info = $hostdata{$index};
	last if $info;
    }
    $info;
}

sub resolve {
    local($hostname) = @_;
    undef(local($lat, $long, $place, $info));
    
    if (($info = &longestmatch($hostname)) && $info ne ";") {
	($lat, $long, $place) = split(';',$info);
#	printf JUNK "resolve host %s lat %g long %g city %s\n", $hostname, $lat, $long, $place;
    } else {		
	printf JUNK "no place for $hostname at $place\n";
	return (-1, -1, -1);
    }			
    ($lat, $long, $place);
}

close(JUNK);
