#!/usr/bin/perl
#
# Integrate Geomview with 3D data on the Web.
# based on weboogl.perl by Tamara Munzner, Paul Burchard, and Stuart Levy
# Feb 1996 Tamara Munzner
# Publically distributable according to the terms of
# the GNU General Public License.
#
# This script runs as a Geomview external module.
#	"geomview -run mbone.perl <filename>"
#
########################################################################
########################################################################
## Main program.
##
## After setup, listen for user picks from Geomview, and possibly for
## new OOGL data files from Mosaic.
########################################################################
########################################################################

package main;

# Deal with signals...
#
# Clean up on exit.
$SIG{'HUP'} = 'exit_cleanup';

# Deal with I/O...
#
# Flush all buffers promptly. 
select(STDERR);
$| = 1;
select (STDOUT);
$| = 1;
#
# Use TOGV instead of just plain STDOUT for later flexibility.
open(TOGV,">-");
select(TOGV);
$| = 1;

# Prepare Geomview...
#
# Tell Geomview we want to hear about pick events.
print TOGV "(interest (pick primitive * * nil nil nil * nil nil nil))\n";
#
# Tell Geomview to tell us when changing from Euclidean to hyperbolic space.
# 
$space = "euclidean";
#print TOGV "(space $space)\n";
print TOGV "(interest (space))\n";
#
# We care when the windowsize changes
# 
print TOGV "(interest (camera-reset))\n";
#
# Define alphabet handles in Geomview (for labels).
&loadfont();
#
# Turn off bounding boxes, normalization, and pick feedback.
print TOGV "(bbox-draw allgeoms no)\n";
print TOGV "(normalization allgeoms none)\n";
#print TOGV "(look-encompass-size 1.0 100 0.1 4.0)\n";
print TOGV "(geometry selection {LIST})(merge-ap selection {appearance {linewidth 8 material {edgecolor .2 .8 .2}}})\n";
#print TOGV "(load globe.oogl geometry)\n";

# Put up control panel
$actionmode = "about";
&setup_controls;

# Main loop: interact with Geomview and 2D browser
#
# When Geomview sends back a pick event, get the associated COMMENT
#	objects which have embedded MBONE info.

$gvfailcount = 20;
$leftovers = "";
while(($rest = <STDIN>) || --$gvfailcount >= 0) {
    $gvfailcount = 20 if defined($rest);

    while ( $rest ) {
	# keep parsing until we've used up the line
	$line = $leftovers . $rest;
	if ($line =~ /\(\s*camera-reset([^\)]*)\)(.*)/) {
	    print TOGV "(merge camera controls {fov $fov})\n";	    
	    $rest = $2; $leftovers = ""; redo;
	}
	if ($line =~ /\(\s*backcolor\s+focus\s+([\d\.\s]*)\s*\)(.*)/) {
	    # unflash background to the color the user set
	    $idle_back = "(backcolor allcams $1)\n"; 
	    $rest = $2; $leftovers = ""; redo;
	} #end of backcolor processing
	if ($line =~ /\(\s*space\s+(\S*)\s*\)(.*)/) {
	    # adjust the control panel to look right in either space
	    $space = $1;
	    if ($space eq "hyperbolic") {
		$fov = $hypfov;
		$recenter = "(echo \"\n\")";
	    } elsif ($space eq "euclidean") {
		$fov = $eucfov;
		$recenter="\n(progn (new-center World)(new-center c0)(echo \"\n\"))\n";
	    }
	    print TOGV "(merge camera controls {fov $fov})\n";
	    $rest = $2; $leftovers = ""; redo;
	} #end of space processing


	if ( $line =~ /\(pick\s+(\S*)\s+(\S*)\s+\(([^\)]*)\)\s+[^\(]*\(([^\)]*)(.*)/ ) {
	    $coordsys = $1;
	    $id = $2;
            $vert = $3;
	    $primarray = $4;
            if ($id eq "controls") {
                local($x,$y,$z,$w) = split(' ',$vert);
		if ($x < 0) {
		    $actionmode = "follow";
		    print TOGV "$aboutoff $followon\n";
	        } elsif ($x > 0) {
		    $actionmode = "about";
		    print TOGV "$abouton $followoff\n";
                }
            } else {
	    # $primarray is the "pick path", a list of integers
	    # which specify which subpart of the hierarchical OOGL
	    # object was picked. Descent into a complex object
	    # (LIST or INST) adds a new integer to the path. Traversal of
	    # simple objects increments the counter at the current
	    # level. 
	       print TOGV "(write-comments - $id ($primarray))\n";
            }
	    $rest = $5; $leftovers = "";redo;
	} # end of pick processing
	if ( $line =~ 
	    /{\s*COMMENT\s+SRC\s+MBONE\s*{\s*\"\s*(src\s+([^\s]*)\s[^\"]*)\"\s*}\s*}{\s*COMMENT\s+DST\s+MBONE\s*{\s*\"\s*(dst\s+([^\s]*)\s[^\"]*)\"\s*}\s*}(.*)/ ) {
	    $srcip = $2;
	    $dstip = $4;
	    $src = $1;
	    $dst = $3;
	    &genlabels($src,"url");
	    &genlabels($dst,"title");
	    $handle = join("/",$srcip,$dstip);
#	    print STDERR "handle is $handle\n";
	    print TOGV "(geometry selection { : $handle })\n";
#	    print STDERR "got MBONE thesrc $src thedst $dst\n";
	    $rest = $5; $leftovers = ""; redo; 
	}

	if ( $line =~ /\(\s*(({\s*COMMENT.*\s+(HREF\s*{\s*[^{}]*}|HTITLE\s*{\s*\"[^\"]*\"\s*})\s*})*\s*\))(.*)/ ) {
	    # get the *entire* string, delimited by parens
	    # note that there may be parens embedded in quoted strings
	    #  in HTITLE info, don't be fooled by these...

	    # Note that the pick might match more than one comment.
	    # The right choice depends on the structure of the
	    # WebOOGL object and the intent of a particular
	    # application. Given the structure of the demo objects,
	    # the "leaf" comment comes last in the string. 
	    $urllevel = 0;
	    $titlelevel = 0;
	    @urlcomments = "";
	    @titlecomments = "";
	    $nextcomment = $1;
	    $rest = $2; $leftovers = "";
	    while ( $nextcomment =~ 
	    s/(\s*{\s*COMMENT.*\s+(MBONE\s*{\s*[^{}]*}|HREF\s*{\s*[^{}]*}|HTITLE\s*{\s*\"[^\"]*\"\s*})\s*})// ) {
		$firstcomment = $2;
		$firstcomment =~ /(\S*)\s+{\s*\"?(.*)\"?}$/;
		$commenttype = $1;
		$commentinfo = $2;
		if ($commenttype =~ /[Hh][Rr][Ee][Ff]/) {
		    @urlcomments[$urllevel++] = $commentinfo;
		} elsif  ($commenttype =~ /[Hh][Tt][Ii][Tt][Ll][Ee]/) {
		    @titlecomments[$titlelevel++] = $commentinfo;
		}
	    }
	    $commentstring = @urlcomments[$#urlcomments];
	    $commentstring =~ s/\s+/\ /g;
	    $commentstring =~ s/\s*$//;
	    ($url,$modifiers) = 
		($commentstring =~ /\s*[\"]?([^\s\"]*)[\"]?(.*)/);
#	    $url = $commentstring;
	    redo if ($commentstring eq "");
if ($actionmode eq "follow") {
	       print TOGV "$search_back";
   	       # deal with the URL
	       &url_to_oogl_cached($commentstring, $world_geom);
	       # while we're at it...
	       &genlabels(@titlecomments[$#titlecomments],"title");
	       &genlabels($url, "url");
	       print TOGV "$idle_back";
            } elsif ($actionmode eq "about") {
		print TOGV "$flash\n"; 
		&genlabels($url,"url");
		&genlabels(@titlecomments[$#titlecomments],"title");
	        print TOGV "$abouton\n";
	    }
	    redo;
	} #end of comment processing

	if ($line =~ /\( \)(.*)/) {
	    #picked geom had no embedded comment
	    $rest = $1; redo;
	} 

	$leftovers = $line;
	last;
    } # end of line parsing
} # end of infinite read-respond loop
# end of main program
&exit_cleanup();

sub exit_cleanup {
	exit(0);
}

sub die {
	local($string) = @_;
	die $string;
}

########################################################################
# Provide fixed-width 3D vector font for labels (so far just numerals
# and uppercase letters).  Characters are accessed with handles equal
# to themselves.
########################################################################

sub loadfont {

	print TOGV "(read geometry { 
# alphabet.vect
#
# A list of handles defining the uppercase letters of
# the Roman alphabet in a fixed-size 32-by-64 font
# (40-unit spacing is recommended).
#
# Read it into Geomview like this:
#
#	(read geometry { < alphabet.vect })
#
# Then you can write out uppercase strings like this:
#
#	(geometry HI { LIST
#		{ INST geom { : H } transform { 1 0 0 0  0 1 0 0  0 0 1 0  0 0 0 1 } }
#		{ INST geom { : I } transform { 1 0 0 0  0 1 0 0  0 0 1 0  40 0 0 1 } }
#	})
#

{ LIST
{ define / { VECT 1 2 0  2 0  0 0 0  16 64 0}}
{ define : { VECT 2 4 0  2 2 0 0  16 4 0  16 24 0  16 40 0  16 60 0 }}
{ define . { VECT 1 2 0  2 0  16 0 0  16 8 0}}
{ define _ { VECT 1 2 0  2 0  0 0 0 32 0 0}}
{ define - { VECT 1 2 0  2 0  0 32 0 32 32 0}}
{ define A { VECT 2 5 0  3 2  0 0
	0 0 0  16 64 0  32 0 0
	8 32 0  24 32 0 } }
{ define B { VECT 2 12 0  7 5  0 0
	0 0 0  0 64 0  16 64 0  28 56 0  28 44 0  20 36 0  0 36 0
	20 36 0  32 24 0  32 8 0  24 0 0  0 0 0 } }
{ define C { VECT 1 8 0  8  0
	32 56 0  24 64 0  8 64 0  0 56 0  0 8 0  8 0 0  24 0 0  32 8 0 } }
{ define D { VECT 1 7 0  7  0
	0 0 0  0 64 0  20 64 0  32 52 0  32 8 0  24 0 0  0 0 0 } }
{ define E { VECT 2 6 0  4 2  0 0
	32 64 0  0 64 0  0 0 0  32 0 0
	0 36 0  24 36 0 } }
{ define F { VECT 2 5 0  3 2  0 0
	0 0 0  0 64 0  32 64 0
	0 36 0  24 36 0 } }
{ define G { VECT 1 10 0  10  0
	32 56 0  24 64 0  8 64 0  0 56 0  0 8 0  8 0 0  24 0 0  32 8 0  32 36 0  16 36 0 } }
{ define H { VECT 3 6 0  2 2 2  0 0 0
	0 0 0  0 64 0
	32 0 0  32 64 0
	0 36 0  32 36 0 } }
{ define I { VECT 3 6 0  2 2 2  0 0 0
	4 64 0  28 64 0
	16 64 0  16 0 0
	4 0 0  28 0 0 } }
{ define J { VECT 2 8 0  2 6  0 0
	12 64 0  36 64 0
	28 64 0  28 8 0  20 0 0  8 0 0  0 8 0  0 20 0 } }
{ define K { VECT 2 5 0  2 3  0 0
	0 0 0  0 64 0
	24 64 0  0 36 0  32 0 0 } }
{ define L { VECT 1 3 0  3  0
	0 64 0  0 0 0  32 0 0 } }
{ define M { VECT 1 5 0  5  0
	0 0 0  0 64 0  16 36 0  32 64 0  32 0 0 } }
{ define N { VECT 1 4 0  4  0
	0 0 0  0 64 0  32 0 0  32 64 0 } }
{ define O { VECT 1 9 0  9  0
	24 0 0  8 0 0  0 8 0  0 52 0  8 64 0  24 64 0  32 52 0  32 8 0  24 0 0 } }
{ define P { VECT 1 7 0  7  0
	0 0 0  0 64 0  16 64 0  28 56 0  28 44 0  20 36 0  0 36 0 } }
{ define Q { VECT 2 11 0  9 2  0 0
	24 0 0  8 0 0  0 8 0  0 52 0  8 64 0  24 64 0  32 52 0  32 8 0  24 0 0
	20 16 0  36 0 0 } }
{ define R { VECT 2 9 0  7 2  0 0
	0 0 0  0 64 0  16 64 0  28 56 0  28 44 0  16 36 0  0 36 0
	16 36 0  32 0 0 } }
{ define S { VECT 1 12 0  12  0
	32 56 0  24 64 0  8 64 0  0 56 0  0 44 0  8 36 0  20 36 0  32 24 0  32 8 0  24 0 0  8 0 0  0 8 0 } }
{ define T { VECT 2 4 0  2 2  0 0
	0 64 0  32 64 0
	16 0 0  16 64 0 } }
{ define U { VECT 1 6 0  6  0
	0 64 0  0 8 0  8 0 0  24 0 0  32 8 0  32 64 0 } }
{ define V { VECT 1 3 0  3  0
	0 64 0  16 0 0  32 64 0 } }
{ define W { VECT 1 5 0  5  0
	0 64 0  4 0 0  16 28 0  28 0 0  32 64 0 } }
{ define X { VECT 2 4 0  2 2  0 0
	0 64 0  32 0 0
	0 0 0  32 64 0 } }
{ define Y { VECT 2 5 0  2 3  0 0
	0 64 0  16 36 0
	32 64 0  16 36 0  16 0 0 } }
{ define Z { VECT 1 4 0  4  0
	0 64 0  32 64 0  0 0 0  32 0 0 } }
{ define 0 { VECT 2 11 0  2 9  0 0
	0 20 0  28 56 0
	24 0 0  8 0 0  0 8 0  0 48 0  8 64 0  24 64 0  32 52 0  32 12 0  24 0 0 } }
{ define 1 { VECT 2 5 0  2 3  0 0
	4 0 0  28 0 0
	4 44 0  16 64 0  16 0 0 } }
{ define 2 { VECT 1 7 0  7  0
	0 56 0  8 64 0  24 64 0  32 56 0  32 40 0  0 0 0  32 0 0 } }
{ define 3 { VECT 2 13 0  2 11  0 0
	8 36 0  24 36 0
	0 56 0  8 64 0  24 64 0  32 56 0  32 44 0  24 36 0  32 24 0  32 8 0  24 0 0  8 0 0  0 8 0 } }
{ define 4 { VECT 2 5 0  2 3  0 0
	28 64 0  28 0 0
	12 64 0  0 36 0  36 36 0 } }
{ define 5 { VECT 1 9 0  9  0
	28 64 0  4 64 0  0 36 0  24 36 0  32 24 0  32 8 0  24 0 0  8 0 0  0 8 0 } }
{ define 6 { VECT 1 12 0  12  0
	32 56 0  24 64 0  8 64 0  0 56 0  0 8 0  8 0 0  24 0 0  32 8 0  32 28 0  24 36 0  8 36 0  0 28 0 } }
{ define 7 { VECT 1 4 0  4  0
	0 56 0  4 64 0  32 64 0  8 0 0 } }
{ define 8 { VECT 1 16 0  16  0
	20 36 0  28 44 0  28 56 0  20 64 0  12 64 0  4 56 0  4 44 0  12 36 0  20 36 0  32 24 0  32 8 0  24 0 0  8 0 0  0 8 0  0 24 0  12 36 0 } }
{ define 9 { VECT 1 12 0  12  0
	32 44 0  24 36 0  8 36 0  0 44 0  0 56 0  8 64 0  24 64 0  32 56 0  32 8 0  24 0 0  8 0 0  0 8 0 } }
}

})"; # end of Gcl command sent to Geomview
} # end of Perl subroutine

sub setup_controls {
    # since the camera is 3 units away by default
    $dist = 1.1; 
    # looks reasonable
    $frameaspect = 20;
    @bar = split(/\s+/,`xdpyinfo | awk ' /dimensions/ {print  $2 }'`);
    @screensize = split(/x/,@bar[2],2);
    @size = (@screensize[0],@screensize[0]/$frameaspect);
    @size[1] = int(@size[1]);
    $boxsize = $frameaspect*$dist;
    # correct FOV for hyp space with otherwise default camera
    $hypfov = .366;
    $eucfov = 2.2;
    $fov = $eucfov if (!defined($fov));
    print TOGV "
(progn
(window default {noborder size $screensize[0] $size[1] position 0 $screensize[0] 0 $size[1]})
(camera controls {
camtoworld transform {
             1             0             0             0
             0             1             0             0
             0             0             1             0
             0             0		 3             1
}
        perspective 0  stereo 0
        fov 2.2
        frameaspect $frameaspect
        focus 3
        near 0.1
        far 100
})
(scene controls
{appearance {linewidth 2 +edge}
{LIST
{ : title }
{ : url }
{ : fbutton }
{ : abutton }
}})
(window default { position 0 300 0 300})
)
";

$fbutton="{QUAD -$boxsize -$dist 0 0 -$dist 0 0 $dist 0 -$boxsize $dist 0} ";
$abutton = "{QUAD $boxsize -$dist 0 0 -$dist 0 0 $dist 0 $boxsize $dist 0} ";

$bluecol = " appearance {-edge material *diffuse 1 1 1 } ";
$aboutoff = "(read geometry {define abutton {$graycol $abutton}})";
$flash = "(read geometry {define abutton {$flashcol $abutton}})";
$bothon = "(read geometry {define fbutton {$bluecol $fbutton}})(read geometry {define abutton {$bluecol $abutton}})";
print TOGV "$bothon\n";
}

sub genlabels {
    local($str, $tier) = @_;
    local (@str);
    # alphabet is 32x64
    local ($spacing) = $dist*0.5;
    local ($labelscale) = $dist*.5/64;
#    @labeladj[0] = -$boxsize+($frameaspect*.05);
    @labeladj[0] = +$boxsize-($frameaspect*.05);
    @labeladj[1] = $dist*.25 - $dist*.9;
    @labeladj[2] = .01;
    local($x, $y, $z) = @labeladj;

    if ($tier eq "title") {$y += $dist;}
    print "(read geometry { define $tier {LIST\n";
    $str =~ tr/a-z/A-Z/;
    @str = split(//,$str);
    @str = reverse(@str);
    for ($i = 0; $i <= $#str; $i++) {
	if (@str[$i] =~ /\w/ || @str[$i] =~ /[\/:_.-]/) {
	    printf "
\t{INST geom {: %s } transform { 
$labelscale 0 0 0  0 $labelscale 0 0  0 0 $labelscale 0  $x $y $z 1 }}\n", 
	    @str[$i];
	}	
	$x -= $spacing;
    }
    print "}})\n";
}

1;
