#!/usr/sbin/perl

# mtrace2edge.perl
# canonicalize mtrace to edge format:
# lines of quoted name/value pairs separated by commas, all enclosed in parens
# i.e.
# ( "srchost=uc.cache.nlanr.net", "srcip=141.142.121.5", .... )
# 5/96
# tamara munzner

# optional (but preferable) input: name of machine mtrace ran from.
# then we can tack on its domain to machines in our domain, who
# would otherwise be unresolved. plus, of course, we get the hostname

if ($ARGV[0]) {
    $starthost = $ARGV[0];
    @tmp = split('\.',$starthost);
    shift(@tmp);
    $startdomain = join('.', @tmp);
}

while (<STDIN>){
    chop;
    @line = split(' ');
#print "### ", join('%',@line), "%\n";
    if ($lostit == 1) {
	shift(@line); # toss number
	while ($line[0] eq "*") {
	    $miss++;
	    shift(@line);
	    if ($miss > 3) {
		$lostit = 1;
		next;
	    }
	}
	$dsthost = "\L$line[0]\E" if $line[0] =~ /[a-z]|[A-Z]/;
	$dstp = substr($line[1], 1, length($line[1]) -2);
	&printedge;
	$srchost = "\L$line[0]\E" if $line[0] =~ /[a-z]|[A-Z]/;
	$srcip = substr($line[1], 1, length($line[1]) -2);
	$lostit = 0;
    } elsif ($line[0] eq "Mtrace" && $line[1] eq "from") {
	$srcip = $line[2];
	$lostit = 0;
	$go = 1;
	$srchost = $starthost if $starthost;
    } elsif ($line[0] eq "Querying") {
	# just keep going, skip this line
    } elsif ($line[0] =~ /Waiting to accumulate/) {
	$lostit = 0;
	$go = 0;
    } elsif ($go == 1) {
	undef(local(@dsthost, $dstpieces));
	shift(@line); # toss number
	$miss = 0;
        # 3 stars in a row means no data for that hop
	while ($line[0] eq "*") {
	    $miss++;
	    shift(@line);
	    if ($miss == 3) {
		$lostit = 1;
		next;
	    }
	}
	$dsthost = "\L$line[0]\E";

	# if there's no domain, it's part of the same one as the caller
	@dstpieces = split('\.',$dsthost);
	$dsthost = $dsthost . "." . $startdomain if $#dstpieces < 1;
	$dstip = substr($line[1], 1, length($line[1])-2);

	# for now, toss away this link: 
	#  there's just an IP address, no hostname.
	# we'll just have an edge that skips over that node 
	#  and goes on to the next.
	# eventually pass this along after we implement the ip resolver.
	if ($dstpieces[0] eq "?") {
	    print STDERR "tossed $srchost $dstip\n";
	    next;
	}
	&printedge;
	$srchost = $dsthost;
	$srcip = $dstip;
    } else {
	$go = 0;
    }
}


sub printedge {
    $str = "(";
    $str .= " \"edgetype=skip\"," if $lostit == 1;
    $str .= " \"srchost=$srchost\"," if $srchost =~ /[a-z]|[A-Z]/;
    $str .= " \"dsthost=$dsthost\"," if $dsthost =~ /[a-z]|[A-Z]/;
    $str .= " \"srcip=$srcip\", \"dstip=$dstip\")\n";
    # don't print out when line is blank or src==dst
    printf $str 
	if $dstip =~ /[0-9]/ && 
	    $srchost ne $dsthost && $srcip ne $dstip; 
}
